// ---------------------------------------------------------------------------
// Created by Francisco Malpartida on 16/04/11.
// Copyright 2011 - Under creative commons license:
//        Attribution-NonCommercial-ShareAlike CC BY-NC-SA
//
// This software is furnished "as is", without technical support, and with no 
// warranty, express or implied, as to its usefulness for any purpose.
//
// Thread Safe: No
// Extendable: Yes
//
// @file <filename>.H
// File description here
// 
// @brief <very brief class description>
//
// @author F. Malpartida
// ---------------------------------------------------------------------------

// Include header files
// ----------------------------------------------------------------------------
#include "Arduino.h"
#include <inttypes.h>
#include "Wire.h"
#include "thermistor.h"
#include "lm75.h"

// Definitions
// ----------------------------------------------------------------------------
#define CONF_REG     1
#define TEMP_REG     0
#define THYST_REG    2
#define TOS_REG      3

#define SHUTDOWN_BIT 0

#define TEMP_BYTES   2

// Static  member variable definitions
// ----------------------------------------------------------------------------

// Static file scope variable definitions
// ----------------------------------------------------------------------------

// Private support functions
// ----------------------------------------------------------------------------
static uint8_t setPointerRegister ( uint8_t address, byte pointer )
{
   // Set the address of the device to write to
   // Write to the device corresponding pointer register
   // 
   Wire.beginTransmission ( address );
   Wire.write ( pointer );
   return ( Wire.endTransmission () );
}


static float rawToCesius ( int temp )
{
   if ( (temp & 0x0800) == 0 )
   {
      return ( ( temp ) * 0.125 );
   }
   else
   {
      return ( -( (~(temp)+1) * 0.125) );
   }   
   
}

// CLASS METHODS
// ----------------------------------------------------------------------------

// Constructors:
// ---------------------------------------------------------------------------
lm75::lm75( uint8_t address )
{
   _address = address;
}

lm75::lm75 ( void )
{
   _address   = 0; 
   _mode      = NORMAL; // this is the nominal mode in which the device powers up
   _confReg   = 0x0;
   _available = false;
}

// Destructor:
// ---------------------------------------------------------------------------
lm75::~lm75() { };



// Functions: modifiers (set), selectors (get)
// ---------------------------------------------------------------------------
void lm75::init ( void )
{
   _available = false;
   
   Wire.begin();
   if ( modeSet(NORMAL) == 0 )
   {
      _available = true;
   }
}

bool lm75::isAvailable ( void )
{
   return (_available);
}


int lm75::modeSet ( t_mode mode )
{
   int retVal = 1;
   
   // Only change mode if required, but will be used to see if there is a
   // thermistor conected to the particular address
   // -------------------------------------------------------------------
   switch (mode) 
   {
      case NORMAL:
         retVal = configureNormalMode ( );
         break;
      case SHUTDOWN:
         retVal = configureShutdownMode( );
         break;

      default:
         break;
   }
   
   // Change the mode if the transaction when well
   // --------------------------------------------
   if ( retVal == 0 )
   {
      _mode = mode;
   }
   return ( retVal );
}


float lm75::readTemperature ( void )
{
   int   rawTemp;
   
   if ( _mode == SHUTDOWN )
   {
      configureNormalMode ();
   }
   readRaw( rawTemp );
   if ( _mode == SHUTDOWN )
   {
      configureShutdownMode();
   }
   
   return ( rawToCesius(rawTemp >> 2) );
}



int lm75::readRaw ( int &temperature )
{
   byte readBytes      = 0;
   
   temperature = 0;
      
   if (_available)
   {
      
      Wire.requestFrom ( _address, uint8_t(TEMP_BYTES) );
   
      while ( Wire.available () && readBytes < TEMP_BYTES )
      {
         temperature = temperature << 8;
         temperature |= Wire.read ();
         readBytes++;
      }
   
      temperature = temperature >> 3;
   
      //Serial.print ("Read value: ");
      //Serial.println ( temperature );
      //Serial.println ( int(readBytes) );
   }
   return ( readBytes );
}

/**
 Write description of function here. 
 The function should follow these comments.
 
 The function arguments listed with "param" will be compared
 to the declaration and verified.
 
 @param[in]     _inArg1 Description of first function argument.
 @param[out]    _outArg2 Description of second function argument.
 @param[in,out] _inoutArg3 Description of third function argument.
 
 @return Description of returned value.
 */
uint8_t lm75::addressGet ( void )
{
   return ( _address );  
}

void lm75::addressSet ( uint8_t address )
{
   _address = address;
}

t_mode lm75::modeGet ( void )
{
   return (_mode);
}

// PRIVATE METHODS
// -----------------------------------------------------------------------------
// return 0: ACK
int lm75::configureNormalMode ( )
{
   int retVal = 1;
   
   _confReg &= ~(1 << SHUTDOWN_BIT);    // Clear bit 0 to enter normal mode
   Wire.beginTransmission ( _address );
   Wire.write ( CONF_REG );
   Wire.write ( _confReg );
   
   // If the transmission went well return the read pointer to temperature
   // register
   // -----------------------------------------------------------------------
   if ( Wire.endTransmission() == 0 )
   {
      retVal = setPointerRegister( _address, TEMP_REG);
   }
   
   return ( retVal );
}

// return 0: ACK
int lm75::configureShutdownMode ( )
{
   int retVal = 1;
   
   _confReg |= (1 << SHUTDOWN_BIT);      // Set bit 0 to enter shutdown mode
   Wire.beginTransmission ( _address );
   Wire.write ( CONF_REG );
   Wire.write ( _confReg );
   
   // If the transmission went well return the read pointer to temperature
   // register
   // -----------------------------------------------------------------------      
   if ( Wire.endTransmission() == 0 )
   {
      retVal = setPointerRegister( _address, TEMP_REG);
   }
   
   return ( retVal );
}


// Attributes visible by scope of instantiation and use

